using System;
using System.Collections.Generic;
using System.Text;

namespace Engine
{
    public class RoomClearingExperiment : MultiAgentExperiment
    {
		
        public RoomClearingExperiment()
            : base()
        {
            homogeneousTeam = false;

            scriptFile = "";
            numberRobots = 1;

            sensorDensity = 21;
            timestep = 0.2f;

            evaluationTime = 60;

            explanatoryText = "Multi-Agent Room Clearing Experiment";
            fitnessFunctionName = "CoverageFitness";
            robotModelName = "RoomClearingRobotModel";

            initialized = false;
            running = false;

            gridCollision = true;
        }

        public RoomClearingExperiment(MultiAgentExperiment exp)
        {
            gridCollision = exp.gridCollision;

            substrateDescriptionFilename = exp.substrateDescriptionFilename;
            adaptableANN = exp.adaptableANN;
            modulatoryANN = exp.modulatoryANN;
            normalizeWeights = exp.normalizeWeights;
            headingNoise = exp.headingNoise;
            effectorNoise = exp.effectorNoise;
            sensorNoise = exp.sensorNoise;

            timesToRunEnvironments = exp.timesToRunEnvironments;
            //TODO: Maybe this should be part of environment, different environments take different amounts of time to evaluate?
            evaluationTime = exp.evaluationTime;
            initialized = false;

            timestep = exp.timestep;
            explanatoryText = exp.explanatoryText;

            scriptFile = exp.scriptFile;

            robotModelName = exp.robotModelName;

            fitnessFunctionName = exp.fitnessFunctionName;
            behaviorCharacterizationName = exp.behaviorCharacterizationName;
            environmentName = exp.environmentName;


            bestGenomeSoFar = exp.bestGenomeSoFar;

            numberRobots = exp.numberRobots;

            homogeneousTeam = exp.homogeneousTeam;

            overrideDefaultSensorDensity = exp.overrideDefaultSensorDensity;
            sensorDensity = exp.sensorDensity;

            useScript = exp.useScript;

            running = exp.running;

            inputs = exp.inputs;
            outputs = exp.outputs;

            agentsVisible = exp.agentsVisible;
            agentsCollide = exp.agentsCollide;

            genome = exp.genome;

            multipleEnvironments = exp.multipleEnvironments;

            evolveSubstrate = exp.evolveSubstrate;
            modulatory = exp.modulatory;

            elapsed = exp.elapsed;

            populationSize = exp.populationSize;

            overrideTeamFormation = exp.overrideTeamFormation;
            group_orientation = exp.group_orientation;
            group_spacing = exp.group_spacing;
            robot_heading = exp.robot_heading;

        }

        protected internal override bool runEnvironment(Environment e, float delta, float headingNoise, float sensorNoisde, float effectorNoise)
        {
            bool value = base.runEnvironment(e, delta, headingNoise, sensorNoisde, effectorNoise);

            foreach (Robot r in robots)
            {
                if (r.autopilot && e.AOIRectangle.Contains((int)r.location.x, (int)r.location.y))
                    r.autopilot = false;
            }

            return value;
        }
    }
}
