using System;
using System.Collections.Generic;
using System.Text;

namespace Engine
{
    class CoverageFitness : IFitnessFunction
    {
        public double accum;
        public CoverageFitness()
        {
            accum = 0.0;
        }
        #region IFitnessFunction Members

        string IFitnessFunction.name
        {
            get { return this.GetType().Name; }
        }

        string IFitnessFunction.description
        {
            get { return "Fitness Awarded for Coverage of Map"; }
        }

        double IFitnessFunction.calculate(SimulatorExperiment engine, Environment environment)
        {
        
            return accum;
        }

        void IFitnessFunction.update(SimulatorExperiment Experiment, Environment environment)
        {
            collision_grid grid = ((GridCollision)((MultiAgentExperiment)Experiment).collisionManager).grid;

            if (!(Experiment.timeSteps % 5 == 0))
            {
                grid.decay_viewed(0);
                return;
            }

            int dim = grid.coarseness;
            for (int x = 0; x < dim; x++)
            {
                for (int y = 0; y < dim; y++)
                {
                    int gx = (int)((double)x * grid.gridx) + (int)(grid.gridx / 2.0);
                    int gy = (int)((double)y * grid.gridy) + (int)(grid.gridy / 2.0);
                    if ((environment.AOIRectangle.Contains(gx, gy)))
                        accum += grid.grid[x, y].viewed;
                }
            }

            grid.decay_viewed(0);
        }

        void IFitnessFunction.reset()
        {
            accum = 0.0;
        }

        #endregion
    }
}
