using System;
using System.Collections.Generic;
using System.Text;
using SharpNeatLib.NeuralNetwork;
using SharpNeatLib;
using System.Drawing;

namespace Engine
{

    //Supports basic rangefinders and pie-slice sensors
    //TODO make this class abstract
    public abstract class Robot : SimulatorObject
    {
        protected AgentBrain agentBrain;
		public bool collisionPenalty =false;
        public int id;        //Robot ID
        public List<ISensor> sensors = new List<ISensor>();
        public double velocity, heading, radius = 5;
        public int steps;
        public int marker;
        public Circle2D circle;
        public Point2D old_location = new Point2D();
        public bool collide_last;
        public bool autopilot;

        protected float headingNoise;
        protected float effectorNoise;
        protected float sensorNoise;
        protected float timeStep;

        //z-stack coordinate
        public float zstack;
 
        //Reference to current environment
        public Environment environment;

        public bool stopped = false;
        public bool disabled = false;

        public int collisions = 0;
  
        public Robot() { dynamic=true; }

          public void init(int id, double nx, double ny, double dir, 
            AgentBrain agentBrain, Environment environment,
            float sensorNoise, float effectorNoise, float headingNoise, float timeStep)
        {
            steps = 0;

            this.id = id;
    	    radius = defaultRobotSize();
            location = new Point2D(nx, ny);
            circle = new Circle2D(location, radius);
            old_location = new Point2D(location);
           
            heading = dir;
            velocity = 0.0;
            collide_last = false;
            this.timeStep = timeStep;
            this.environment = environment;
            this.agentBrain = agentBrain;
            this.sensorNoise = sensorNoise;
            this.effectorNoise = effectorNoise;
            this.headingNoise = headingNoise;
            populateSensors();
            
        }

        public override void update()
        {
            throw new NotImplementedException();
        }

        //Called when robot collides
        public virtual void onCollision()
        {
            collisions++;
			undo();
			
			if (collisionPenalty && collisions > 0)
            {
                disabled = true;
                stopped = true;
            }
            //if (robot.collisions > 1)
            //{
            //    robot.disabled = true;
            //    robot.stopped = true;
            //}
            //else
            //{
            //    robot.health = (1.0 / ((double)robot.collisions + 1.0));
            //    robot.undo();
            //}
        }

        public double noisyHeading()
        {
            return heading + (timeStep * 1.28 * (Utilities.random.NextBool() ? 1 : -1) * Utilities.random.Next(0, (int)headingNoise) / 100.0);
        }

        public void updateSensors(Environment env, List<Robot> robots,CollisionManager cm)
        {
			foreach (ISensor sensor in sensors) {
				sensor.update(env, robots,cm);
			}
        }


        public void populateSensors(float[] rangeFinderSensorAngles)
        {
            sensors = new List<ISensor>();
            foreach (double val in rangeFinderSensorAngles)
            {
                sensors.Add(new RangeFinder(val, this, 400.0,this.sensorNoise));
            }
        }
            
        public void updatePosition()
        {
            //record old coordinates
            old_location.x = location.x;
            old_location.y = location.y;

            //update current coordinates (may be revoked if new position forces collision)
            if (!stopped)
            {
                double tempHeading = noisyHeading();
                double dx = Math.Cos(tempHeading);
                double dy = Math.Sin(tempHeading);
                location.x += velocity * dx * timeStep;
                location.y += velocity * dy * timeStep;
            }
        }

        public override void undo()
        {
            location.x = old_location.x;
            location.y = old_location.y;
        }

        //default robot draw method
        public virtual void draw(Graphics g, CoordinateFrame frame)
        {
            Point upperleft = frame.to_display((float)(circle.p.x - radius),(float)(circle.p.y- radius));
			int size = (int)((radius*2)/frame.scale);
			Rectangle r = new Rectangle(upperleft.X,upperleft.Y,size,size);
            if (disabled)
				g.DrawEllipse(EngineUtilities.YellowPen,r);
			else if (collide_last)
                g.DrawEllipse(EngineUtilities.RedPen, r);
            else
                g.DrawEllipse(EngineUtilities.BluePen, r);
			foreach(ISensor sensor in sensors)
			{
				sensor.draw(g,frame);
			}
        }
        
        public abstract float defaultRobotSize();

        public abstract int defaultSensorDensity();

        public abstract void populateSensors();
	
        public abstract void populateSensors(int size);

        public abstract void networkResults(float[] outputs);
        
        public virtual void robotSettingsChanged()
        {
        }

        public virtual void doAction() {}
    }
}
